@@ -3,6 +3,7 @@ Configuration loader using 'git-config'.
 """
 
 import logging
+import typing as ty
 
 from git_pw import utils
 
@@ -11,10 +12,10 @@ LOG = logging.getLogger(__name__)
 
 class Config(object):
 
-    def __init__(self):
-        self._git_config = {}
+    def __init__(self) -> None:
+        self._git_config: ty.Dict[str, str] = {}
 
-    def __getattribute__(self, name):
+    def __getattribute__(self, name: str) -> str:
         # attempt to use any attributes first
         try:
             value = super(Config, self).__getattribute__(name)
