@@ -3,8 +3,8 @@ Patch subcommands.
 """
 
 import logging
-import pty
 import sys
+import os
 
 import arrow
 import click
@@ -58,7 +58,11 @@ def apply_cmd(patch_id, series, deps, args):
 
 @click.command(name='download')
 @click.argument('patch_id', type=click.INT)
-@click.argument('output', type=click.File('wb'), required=False)
+@click.argument(
+    'output',
+    type=click.Path(file_okay=True, writable=True, readable=True),
+    required=False
+)
 @click.option('--diff', 'fmt', flag_value='diff',
               help='Show patch in diff format.')
 @click.option('--mbox', 'fmt', flag_value='mbox', default=True,
@@ -67,8 +71,8 @@ def download_cmd(patch_id, output, fmt):
     """Download patch in diff or mbox format.
 
     Download a patch but do not apply it. ``OUTPUT`` is optional and can be an
-    output path or ``-`` to output to ``stdout``. If ``OUTPUT`` is not
-    provided, the output path will be automatically chosen.
+    output file path or a directory or ``-`` to output to ``stdout``. If
+    ``OUTPUT`` is not provided, the output path will be automatically chosen.
     """
     LOG.debug('Downloading patch: id=%d, format=%s', patch_id, fmt)
 
@@ -76,10 +80,15 @@ def download_cmd(patch_id, output, fmt):
     patch = api.detail('patches', patch_id)
 
     if fmt == 'diff':
-        if output:
-            output.write(patch['diff'])
-            if output.fileno() != pty.STDOUT_FILENO:
-                path = output.name
+        if output and not os.path.isdir(output):
+            if output == '-':
+                output_path = 0  # stdout fd
+            else:
+                output_path = output
+                path = output
+
+            with open(output_path, 'w') as output_file:
+                output_file.write(utils.ensure_str(patch['diff']))
         else:
             # TODO(stephenfin): We discard the 'diff' field so we can get the
             # filename and save to the correct file. We should expose this
@@ -269,8 +278,10 @@ def list_cmd(states, submitters, delegates, hashes, archived, limit, page,
             patch.get('id'),
             arrow.get(patch.get('date')).humanize(),
             utils.trim(patch.get('name')),
-            '%s (%s)' % (patch.get('submitter').get('name'),
-                         patch.get('submitter').get('email')),
+            '%s (%s)' % (
+                patch.get('submitter').get('name'),
+                patch.get('submitter').get('email'),
+            ),
             patch.get('state'),
             'yes' if patch.get('archived') else 'no',
             (patch.get('delegate') or {}).get('username', ''),
